@@ -61,6 +61,13 @@ class TestHTTPUtility(object):
         assert a.best_match(["foo/bar", "bar/foo"], default="foo/bar") == "foo/bar"
         assert a.best_match(["application/xml", "text/xml"]) == "application/xml"
 
+    def test_accept_mime_specificity(self):
+        a = http.parse_accept_header(
+            "text/*, text/html, text/html;level=1, */*", datastructures.MIMEAccept
+        )
+        assert a.best_match(["text/html; version=1", "text/html"]) == "text/html"
+        assert a.best_match(["text/html", "text/html; level=1"]) == "text/html; level=1"
+
     def test_charset_accept(self):
         a = http.parse_accept_header(
             "ISO-8859-1,utf-8;q=0.7,*;q=0.7", datastructures.CharsetAccept
@@ -118,6 +125,14 @@ class TestHTTPUtility(object):
         assert c.private is None
         assert c.to_header() == "no-cache"
 
+    def test_csp_header(self):
+        csp = http.parse_csp_header(
+            "default-src 'self'; script-src 'unsafe-inline' *; img-src"
+        )
+        assert csp.default_src == "'self'"
+        assert csp.script_src == "'unsafe-inline' *"
+        assert csp.img_src is None
+
     def test_authorization_header(self):
         a = http.parse_authorization_header("Basic QWxhZGRpbjpvcGVuIHNlc2FtZQ==")
         assert a.type == "basic"
@@ -371,9 +386,9 @@ class TestHTTPUtility(object):
     def test_is_resource_modified(self):
         env = create_environ()
 
-        # ignore POST
+        # any method is allowed
         env["REQUEST_METHOD"] = "POST"
-        assert not http.is_resource_modified(env, etag="testing")
+        assert http.is_resource_modified(env, etag="testing")
         env["REQUEST_METHOD"] = "GET"
 
         # etagify from data
@@ -427,22 +442,22 @@ class TestHTTPUtility(object):
         assert http.http_date(0) == "Thu, 01 Jan 1970 00:00:00 GMT"
         assert http.http_date(datetime(1970, 1, 1)) == "Thu, 01 Jan 1970 00:00:00 GMT"
 
-    def test_cookies(self):
-        strict_eq(
-            dict(
-                http.parse_cookie(
-                    "dismiss-top=6; CP=null*; PHPSESSID=0a539d42abc001cd"
-                    'c762809248d4beed; a=42; b="\\";"'
-                )
-            ),
-            {
-                "CP": u"null*",
-                "PHPSESSID": u"0a539d42abc001cdc762809248d4beed",
-                "a": u"42",
-                "dismiss-top": u"6",
-                "b": u'";',
-            },
+    def test_parse_cookie(self):
+        cookies = http.parse_cookie(
+            "dismiss-top=6; CP=null*; PHPSESSID=0a539d42abc001cdc762809248d4beed;"
+            ' a=42; b="\\";"; ; fo234{=bar;blub=Blah;'
         )
+        assert cookies.to_dict() == {
+            "CP": u"null*",
+            "PHPSESSID": u"0a539d42abc001cdc762809248d4beed",
+            "a": u"42",
+            "dismiss-top": u"6",
+            "b": u'";',
+            "fo234{": u"bar",
+            "blub": u"Blah",
+        }
+
+    def test_dump_cookie(self):
         rv = http.dump_cookie(
             "foo", "bar baz blub", 360, httponly=True, sync_expires=False
         )
@@ -453,39 +468,37 @@ class TestHTTPUtility(object):
             "Path=/",
             'foo="bar baz blub"',
         }
-
-        strict_eq(
-            dict(http.parse_cookie("fo234{=bar; blub=Blah")),
-            {"fo234{": u"bar", "blub": u"Blah"},
-        )
-
-        strict_eq(http.dump_cookie("key", "xxx/"), "key=xxx/; Path=/")
-        strict_eq(http.dump_cookie("key", "xxx="), "key=xxx=; Path=/")
+        assert http.dump_cookie("key", "xxx/") == "key=xxx/; Path=/"
+        assert http.dump_cookie("key", "xxx=") == "key=xxx=; Path=/"
 
     def test_bad_cookies(self):
-        strict_eq(
-            dict(
-                http.parse_cookie(
-                    "first=IamTheFirst ; a=1; oops ; a=2 ;second = andMeTwo;"
-                )
-            ),
-            {"first": u"IamTheFirst", "a": u"2", "oops": u"", "second": u"andMeTwo"},
+        cookies = http.parse_cookie(
+            "first=IamTheFirst ; a=1; oops ; a=2 ;second = andMeTwo;"
         )
+        expect = {
+            "first": [u"IamTheFirst"],
+            "a": [u"1", u"2"],
+            "oops": [u""],
+            "second": [u"andMeTwo"],
+        }
+        assert cookies.to_dict(flat=False) == expect
+        assert cookies["a"] == u"1"
+        assert cookies.getlist("a") == [u"1", u"2"]
 
     def test_empty_keys_are_ignored(self):
-        strict_eq(
-            dict(
-                http.parse_cookie("first=IamTheFirst ; a=1; a=2 ;second=andMeTwo; ; ")
-            ),
-            {"first": u"IamTheFirst", "a": u"2", "second": u"andMeTwo"},
-        )
+        cookies = http.parse_cookie("spam=ham; duck=mallard; ; ")
+        expect = {"spam": u"ham", "duck": u"mallard"}
+        assert cookies.to_dict() == expect
 
     def test_cookie_quoting(self):
         val = http.dump_cookie("foo", "?foo")
-        strict_eq(val, 'foo="?foo"; Path=/')
-        strict_eq(dict(http.parse_cookie(val)), {"foo": u"?foo"})
+        assert val == 'foo="?foo"; Path=/'
+        assert http.parse_cookie(val).to_dict() == {"foo": u"?foo", "Path": u"/"}
+        assert http.parse_cookie(r'foo="foo\054bar"').to_dict(), {"foo": u"foo,bar"}
 
-        strict_eq(dict(http.parse_cookie(r'foo="foo\054bar"')), {"foo": u"foo,bar"})
+    def test_parse_set_cookie_directive(self):
+        val = 'foo="?foo"; version="0.1";'
+        assert http.parse_cookie(val).to_dict() == {"foo": u"?foo", "version": u"0.1"}
 
     def test_cookie_domain_resolving(self):
         val = http.dump_cookie("foo", "bar", domain=u"\N{SNOWMAN}.com")
@@ -540,16 +553,21 @@ class TestHTTPUtility(object):
         assert "the limit is 512 bytes" in str(w.message)
 
     @pytest.mark.parametrize(
-        "input, expected",
-        [
+        ("samesite", "expected"),
+        (
             ("strict", "foo=bar; Path=/; SameSite=Strict"),
             ("lax", "foo=bar; Path=/; SameSite=Lax"),
+            ("none", "foo=bar; Path=/; SameSite=None"),
             (None, "foo=bar; Path=/"),
-        ],
+        ),
     )
-    def test_cookie_samesite_attribute(self, input, expected):
-        val = http.dump_cookie("foo", "bar", samesite=input)
-        strict_eq(val, expected)
+    def test_cookie_samesite_attribute(self, samesite, expected):
+        value = http.dump_cookie("foo", "bar", samesite=samesite)
+        assert value == expected
+
+    def test_cookie_samesite_invalid(self):
+        with pytest.raises(ValueError):
+            http.dump_cookie("foo", "bar", samesite="invalid")
 
 
 class TestRange(object):
@@ -650,6 +668,18 @@ class TestRange(object):
         assert rv.length == 100
         assert rv.units == "bytes"
 
+    @pytest.mark.parametrize(
+        ("args", "expected"),
+        (
+            ((1, 1, 1), "Mon, 01 Jan 0001 00:00:00 GMT"),
+            ((999, 1, 1), "Tue, 01 Jan 0999 00:00:00 GMT"),
+            ((1000, 1, 1), "Wed, 01 Jan 1000 00:00:00 GMT"),
+            ((2020, 1, 1), "Wed, 01 Jan 2020 00:00:00 GMT"),
+        ),
+    )
+    def test_http_date_lt_1000(self, args, expected):
+        assert http.http_date(datetime(*args)) == expected
+
 
 class TestRegression(object):
     def test_best_match_works(self):
