@@ -27,6 +27,8 @@ def test_basic_routing():
             r.Rule("/", endpoint="index"),
             r.Rule("/foo", endpoint="foo"),
             r.Rule("/bar/", endpoint="bar"),
+            r.Rule("/ws", endpoint="ws", websocket=True),
+            r.Rule("/", endpoint="indexws", websocket=True),
         ]
     )
     adapter = map.bind("example.org", "/")
@@ -36,6 +38,9 @@ def test_basic_routing():
     pytest.raises(r.RequestRedirect, lambda: adapter.match("/bar"))
     pytest.raises(r.NotFound, lambda: adapter.match("/blub"))
 
+    adapter = map.bind("example.org", "/", url_scheme="ws")
+    assert adapter.match("/") == ("indexws", {})
+
     adapter = map.bind("example.org", "/test")
     with pytest.raises(r.RequestRedirect) as excinfo:
         adapter.match("/bar")
@@ -61,6 +66,62 @@ def test_basic_routing():
         adapter.match()
     assert excinfo.value.new_url == "http://example.org/bar/?foo=bar"
 
+    adapter = map.bind("example.org", "/ws", url_scheme="wss")
+    assert adapter.match("/ws", websocket=True) == ("ws", {})
+    with pytest.raises(r.WebsocketMismatch):
+        adapter.match("/ws", websocket=False)
+    with pytest.raises(r.WebsocketMismatch):
+        adapter.match("/foo", websocket=True)
+
+
+def test_merge_slashes_match():
+    url_map = r.Map(
+        [
+            r.Rule("/no/tail", endpoint="no_tail"),
+            r.Rule("/yes/tail/", endpoint="yes_tail"),
+            r.Rule("/with/<path:path>", endpoint="with_path"),
+            r.Rule("/no//merge", endpoint="no_merge", merge_slashes=False),
+        ]
+    )
+    adapter = url_map.bind("localhost", "/")
+
+    with pytest.raises(r.RequestRedirect) as excinfo:
+        adapter.match("/no//tail")
+
+    assert excinfo.value.new_url.endswith("/no/tail")
+
+    with pytest.raises(r.RequestRedirect) as excinfo:
+        adapter.match("/yes//tail")
+
+    assert excinfo.value.new_url.endswith("/yes/tail/")
+
+    with pytest.raises(r.RequestRedirect) as excinfo:
+        adapter.match("/yes/tail//")
+
+    assert excinfo.value.new_url.endswith("/yes/tail/")
+
+    assert adapter.match("/no/tail")[0] == "no_tail"
+    assert adapter.match("/yes/tail/")[0] == "yes_tail"
+
+    _, rv = adapter.match("/with/http://example.com/")
+    assert rv["path"] == "http://example.com/"
+    _, rv = adapter.match("/with/x//y")
+    assert rv["path"] == "x//y"
+
+    assert adapter.match("/no//merge")[0] == "no_merge"
+
+
+def test_merge_slashes_build():
+    url_map = r.Map(
+        [
+            r.Rule("/yes//merge", endpoint="yes_merge"),
+            r.Rule("/no//merge", endpoint="no_merge", merge_slashes=False),
+        ]
+    )
+    adapter = url_map.bind("localhost", "/")
+    assert adapter.build("yes_merge") == "/yes/merge"
+    assert adapter.build("no_merge") == "/no//merge"
+
 
 def test_strict_slashes_redirect():
     map = r.Map(
@@ -143,6 +204,7 @@ def test_basic_building():
             r.Rule("/bar/<float:bazf>", endpoint="barf"),
             r.Rule("/bar/<path:bazp>", endpoint="barp"),
             r.Rule("/hehe", endpoint="blah", subdomain="blah"),
+            r.Rule("/ws", endpoint="ws", websocket=True),
         ]
     )
     adapter = map.bind("example.org", "/", subdomain="blah")
@@ -174,6 +236,11 @@ def test_basic_building():
     assert adapter.build("foo", {}) == "/foo"
     assert adapter.build("foo", {}, force_external=True) == "//example.org/foo"
 
+    adapter = map.bind("example.org", url_scheme="ws")
+    assert adapter.build("ws", {}) == "ws://example.org/ws"
+    assert adapter.build("foo", {}, force_external=True) == "http://example.org/foo"
+    assert adapter.build("foo", {}) == "/foo"
+
 
 def test_long_build():
     long_args = dict(("v%d" % x, x) for x in range(10000))
@@ -358,6 +425,27 @@ def test_http_host_before_server_name():
     assert adapter.build("index") == "http://wiki.example.com/"
 
 
+def test_invalid_subdomain_warning():
+    env = create_environ("/foo")
+    env["SERVER_NAME"] = env["HTTP_HOST"] = "foo.example.com"
+    m = r.Map([r.Rule("/foo", endpoint="foo")])
+    with pytest.warns(UserWarning) as record:
+        a = m.bind_to_environ(env, server_name="bar.example.com")
+    assert a.subdomain == "<invalid>"
+    assert len(record) == 1
+
+
+@pytest.mark.parametrize(
+    ("base", "name"),
+    (("http://localhost", "localhost:80"), ("https://localhost", "localhost:443")),
+)
+def test_server_name_match_default_port(base, name):
+    environ = create_environ("/foo", base_url=base)
+    map = r.Map([r.Rule("/foo", endpoint="foo")])
+    adapter = map.bind_to_environ(environ, server_name=name)
+    assert adapter.match() == ("foo", {})
+
+
 def test_adapter_url_parameter_sorting():
     map = r.Map(
         [r.Rule("/", endpoint="index")], sort_parameters=True, sort_key=lambda x: x[1]
@@ -423,8 +511,10 @@ def test_server_name_interpolation():
     assert adapter.match() == ("alt", {})
 
     env = create_environ("/", "http://%s/" % server_name)
-    adapter = map.bind_to_environ(env, server_name="foo")
-    assert adapter.subdomain == "<invalid>"
+
+    with pytest.warns(UserWarning):
+        adapter = map.bind_to_environ(env, server_name="foo")
+        assert adapter.subdomain == "<invalid>"
 
 
 def test_rule_emptying():
@@ -584,7 +674,7 @@ def test_default_converters():
 def test_uuid_converter():
     m = r.Map([r.Rule("/a/<uuid:a_uuid>", endpoint="a")])
     a = m.bind("example.org", "/")
-    rooute, kwargs = a.match("/a/a8098c1a-f86e-11da-bd1a-00112444be1e")
+    route, kwargs = a.match("/a/a8098c1a-f86e-11da-bd1a-00112444be1e")
     assert type(kwargs["a_uuid"]) == uuid.UUID
 
 
@@ -743,8 +833,10 @@ def test_external_building_with_port_bind_to_environ():
 def test_external_building_with_port_bind_to_environ_wrong_servername():
     map = r.Map([r.Rule("/", endpoint="index")])
     environ = create_environ("/", "http://example.org:5000/")
-    adapter = map.bind_to_environ(environ, server_name="example.org")
-    assert adapter.subdomain == "<invalid>"
+
+    with pytest.warns(UserWarning):
+        adapter = map.bind_to_environ(environ, server_name="example.org")
+        assert adapter.subdomain == "<invalid>"
 
 
 def test_converter_parser():
@@ -895,7 +987,10 @@ def test_server_name_casing():
     env["SERVER_NAME"] = "127.0.0.1"
     env["SERVER_PORT"] = "5000"
     del env["HTTP_HOST"]
-    a = m.bind_to_environ(env, server_name="example.com")
+
+    with pytest.warns(UserWarning):
+        a = m.bind_to_environ(env, server_name="example.com")
+
     with pytest.raises(r.NotFound):
         a.match()
 
@@ -1103,3 +1198,41 @@ def test_build_url_with_arg_keyword():
 
     ret = adapter.build("foo", {"class": "bar"})
     assert ret == "http://example.org/foo/bar"
+
+
+def test_build_url_same_endpoint_multiple_hosts():
+    m = r.Map(
+        [
+            r.Rule("/", endpoint="index", host="alpha.example.com"),
+            r.Rule("/", endpoint="index", host="beta.example.com"),
+            r.Rule("/", endpoint="gamma", host="gamma.example.com"),
+        ],
+        host_matching=True,
+    )
+
+    alpha = m.bind("alpha.example.com")
+    assert alpha.build("index") == "/"
+    assert alpha.build("gamma") == "http://gamma.example.com/"
+
+    alpha_case = m.bind("AlPhA.ExAmPlE.CoM")
+    assert alpha_case.build("index") == "/"
+    assert alpha_case.build("gamma") == "http://gamma.example.com/"
+
+    beta = m.bind("beta.example.com")
+    assert beta.build("index") == "/"
+
+    beta_case = m.bind("BeTa.ExAmPlE.CoM")
+    assert beta_case.build("index") == "/"
+
+
+def test_rule_websocket_methods():
+    with pytest.raises(ValueError):
+        r.Rule("/ws", endpoint="ws", websocket=True, methods=["post"])
+    with pytest.raises(ValueError):
+        r.Rule(
+            "/ws",
+            endpoint="ws",
+            websocket=True,
+            methods=["get", "head", "options", "post"],
+        )
+    r.Rule("/ws", endpoint="ws", websocket=True, methods=["get", "head", "options"])
